
"""
    AbstractQFT

Base type for any QFT. 
"""
abstract type AbstractQFT end

struct Vertex
    p1::AbstractParticle
    p2::AbstractParticle
    p3::AbstractParticle
end

Base.isequal(v1::Vertex, v2::Vertex) = Set([v1.p1, v1.p2, v1.p3]) == Set([v2.p1, v2.p2, v2.p3])

is_valid(v::Vertex) = v.p1 != v.p2 && v.p2 != v.p3


struct QFT <: AbstractQFT
    allowed_vertices::Vector{Vertex}
end

function can_interact(theory::AbstractQFT, v::Vertex)
    for allowed_vertex in theory.allowed_vertices
        if isequal(allowed_vertex, v)
            return true
        end
    end
    return false
end
can_interact(theory::AbstractQFT, p1::AbstractParticle, p2::AbstractParticle, p3::AbstractParticle) = can_interact(theory, Vertex(p1, p2, p3))

QED = QFT([
    Vertex(Electron(), Positron(), Photon()),
    Vertex(Muon(), AntiMuon(), Photon()),
    Vertex(Tauon(), AntiTauon(), Photon())
])
